/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.container.it;

import junit.framework.Assert;
import org.jdtaus.core.container.Container;

/**
 * Testcase for {@code Container} implementations.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: ContainerTest.java 4610 2008-03-10 02:48:18Z schulte2005 $
 */
public class ContainerTest
{
    //--ContainerTest-----------------------------------------------------------

    /** The implementation to test. */
    private Container container;

    /**
     * Gets the {@code Container} implementation tests are performed with.
     *
     * @return the {@code Container} implementation tests are performed
     * with.
     */
    public Container getContainer()
    {
        return this.container;
    }

    /**
     * Sets the {@code Container} implementation to test.
     *
     * @param value the {@code Container} implementation to test.
     */
    public final void setContainer( final Container value )
    {
        this.container = value;
    }

    //-----------------------------------------------------------ContainerTest--
    //--Tests-------------------------------------------------------------------

    /**
     * Tests the {@link Container#getImplementation(Class, String)} method
     * to return the correctly initialized instance of
     * {@link TestImplementation}.
     */
    public void testInstantiation() throws Exception
    {
        assert this.getContainer() != null;

        final TestSpecification spec =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite" );

        Assert.assertTrue( spec.isInitialized() );
    }

    /**
     * Tests the {@link Container#getImplementation(Class, String)} method
     * to return the same instance of {@link TestImplementation} for successive
     * calls.
     */
    public void testSingleton() throws Exception
    {
        assert this.getContainer() != null;

        final TestSpecification spec1 =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite" );

        final TestSpecification spec2 =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite" );

        final TestSpecification spec3 =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite" );

        final TestSpecification child =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite - Child" );

        Assert.assertTrue( spec1 == spec2 );
        Assert.assertTrue( spec2 == spec3 );
        Assert.assertTrue( spec3 == spec1 );
        Assert.assertFalse( child == spec1 );
        Assert.assertFalse( child == spec2 );
        Assert.assertFalse( child == spec3 );
    }

    /**
     * Tests that model properties are correctly provided to implementations.
     */
    public void testProperties()
    {
        final TestSpecification spec1 =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite" );

        final TestSpecification child =
            ( TestSpecification ) this.getContainer().
            getImplementation( TestSpecification.class,
                               "jDTAUS Core Test Suite - Child" );

        this.assertValidProperties( spec1 );
        this.assertValidProperties( child );
    }

    /**
     * Checks the given {@code TestSpecification} to provide the correct
     * property values.
     */
    private void assertValidProperties( final TestSpecification s )
    {
        Assert.assertEquals( s.isBoolean(), true );
        Assert.assertEquals( s.getByte(), ( byte ) 1 );
        Assert.assertEquals( s.getChar(), 'X' );
        Assert.assertTrue( s.getDouble() == 0.1D );
        Assert.assertTrue( s.getFloat() == 0.1F );
        Assert.assertEquals( s.getInt(), 1 );
        Assert.assertEquals( s.getLong(), 1L );
        Assert.assertEquals( s.getShort(), ( short ) 1 );
        Assert.assertEquals( s.isBooleanObject(), Boolean.TRUE );
        Assert.assertEquals( s.getByteObject(), new Byte( ( byte ) 1 ) );
        Assert.assertEquals( s.getCharacterObject(), new Character( 'X' ) );
        Assert.assertEquals( s.getDoubleObject(), new Double( 0.1D ) );
        Assert.assertEquals( s.getFloatObject(), new Float( 0.1F ) );
        Assert.assertEquals( s.getIntegerObject(), new Integer( 1 ) );
        Assert.assertEquals( s.getLongObject(), new Long( 1L ) );
        Assert.assertEquals( s.getShortObject(), new Short( ( short ) 1 ) );
        Assert.assertEquals( s.getStringObject(), "TEST" );
    }

    //-------------------------------------------------------------------Tests--
}
